#ifndef __BOARD_H__
#define __BOARD_H__

//include standard headers for SAM9260
#include "include/AT91SAM9260.h"		//from Atmel
#include "include/lib_AT91SAM9260.h" 	//unofficial one (only few functions)

//global defs
#define EXT_OC          18432000  // External Oscillator frequency
#define MCK             (99328000)  // MCK, master clock for peripherals
#define PPLACK          198656000  // Clock for ARM core

//general purpose switch ("Boot switch")
#define SW_GP_PIO		AT91C_BASE_PIOC
#define SW_GP_MASK		AT91C_PIO_PC15

//on board buzzer
#define BUZZER_PIO		AT91C_BASE_PIOA
#define BUZZER_MASK		AT91C_PIO_PA9

//other switches
#define SW1_MASK		AT91C_PIO_PC13
#define SW2_MASK		AT91C_PIO_PC14
#define SW3_MASK		AT91C_PIO_PC8
#define SW4_MASK		AT91C_PIO_PC10
#define SW_PIO			AT91C_BASE_PIOC

//general purpose Green LED ("Boot LED")
#define LED_GP_PIO		AT91C_BASE_PIOA
#define LED_GP_MASK		AT91C_PIO_PA6

//other LEDS
#define LED_R_PIO		AT91C_BASE_PIOA
#define LED_R_MASK		AT91C_PIO_PA23
#define LED_G_PIO		AT91C_BASE_PIOA
#define LED_G_MASK		AT91C_PIO_PA24
#define LED_Y_PIO		AT91C_BASE_PIOB
#define LED_Y_MASK		AT91C_PIO_PB9
#define LED_B_PIO		AT91C_BASE_PIOB
#define LED_B_MASK		AT91C_PIO_PB8

//interface defs -- to be used in the program

#define SW_GP_PRESSED	((SW_GP_PIO->PIO_PDSR & SW_GP_MASK)==0)
#define SW1_PRESSED		( ( SW_PIO->PIO_PDSR & SW1_MASK ) == 0)
#define SW2_PRESSED		( ( SW_PIO->PIO_PDSR & SW2_MASK ) == 0)
#define SW3_PRESSED		( ( SW_PIO->PIO_PDSR & SW3_MASK ) == 0)
#define SW4_PRESSED		( ( SW_PIO->PIO_PDSR & SW4_MASK ) == 0)
#define SW_PRESSED		( ( SW_PIO->PIO_PDSR & (SW1_MASK | SW2_MASK | SW3_MASK | SW4_MASK) ) != (SW1_MASK | SW2_MASK | SW3_MASK | SW4_MASK) )
#define LED_R_INIT		{LED_R_PIO->PIO_PER = LED_R_MASK; LED_R_PIO->PIO_OER = LED_R_MASK; LED_R_PIO->PIO_OWER = LED_R_MASK;}
#define LED_R_ON		LED_R_PIO->PIO_SODR = LED_R_MASK
#define LED_R_OFF		LED_R_PIO->PIO_CODR = LED_R_MASK
#define LED_R_TOGGLE	LED_R_PIO->PIO_ODSR ^= LED_R_MASK
#define LED_Y_INIT		{LED_Y_PIO->PIO_PER = LED_Y_MASK; LED_Y_PIO->PIO_OER = LED_Y_MASK; LED_Y_PIO->PIO_OWER = LED_Y_MASK;}
#define LED_Y_ON		LED_Y_PIO->PIO_SODR = LED_Y_MASK
#define LED_Y_OFF		LED_Y_PIO->PIO_CODR = LED_Y_MASK
#define LED_Y_TOGGLE	LED_Y_PIO->PIO_ODSR ^= LED_Y_MASK
#define LED_B_INIT		{LED_B_PIO->PIO_PER = LED_B_MASK; LED_B_PIO->PIO_OER = LED_B_MASK; LED_B_PIO->PIO_OWER = LED_B_MASK;}
#define LED_B_ON		LED_B_PIO->PIO_SODR = LED_B_MASK
#define LED_B_OFF		LED_B_PIO->PIO_CODR = LED_B_MASK
#define LED_B_TOGGLE	LED_B_PIO->PIO_ODSR ^= LED_B_MASK
#define LED_G_INIT		{LED_G_PIO->PIO_PER = LED_G_MASK; LED_G_PIO->PIO_OER = LED_G_MASK; LED_G_PIO->PIO_OWER = LED_G_MASK;}
#define LED_G_ON		LED_G_PIO->PIO_SODR = LED_G_MASK
#define LED_G_OFF		LED_G_PIO->PIO_CODR = LED_G_MASK
#define LED_G_TOGGLE	LED_G_PIO->PIO_ODSR ^= LED_G_MASK
#define LED_GP_INIT		{LED_GP_PIO->PIO_PER = LED_GP_MASK; LED_GP_PIO->PIO_OER = LED_GP_MASK; LED_GP_PIO->PIO_OWER = LED_GP_MASK;}
#define LED_GP_ON		LED_GP_PIO->PIO_CODR = LED_GP_MASK
#define LED_GP_OFF		LED_GP_PIO->PIO_SODR = LED_GP_MASK
#define LED_GP_TOGGLE	LED_GP_PIO->PIO_ODSR ^= LED_GP_MASK
#define LEDS_INIT		{LED_R_INIT; LED_G_INIT; LED_Y_INIT; LED_B_INIT;}
#define BUZZER_ON		BUZZER_PIO->PIO_CODR = BUZZER_MASK
#define BUZZER_OFF		BUZZER_PIO->PIO_SODR = BUZZER_MASK
#define BUZZER_INIT		{BUZZER_PIO->PIO_PER = BUZZER_MASK; BUZZER_PIO->PIO_OER = BUZZER_MASK; BUZZER_OFF;}

#endif
