#include "common.h"
#include "xmodem.h"


static uint16_t xmodemNextByteCrc(uint16_t crc, uint8_t nextByte);

static uint8_t blockCounter=1;
static uint8_t xModemStarted;




static uint16_t xmodemNextByteCrc(uint16_t crc, uint8_t nextByte)
{
	uint8_t i;
	crc = crc ^ ((uint16_t)nextByte << 8);
	for(i=0;i<8;i++)
	{
		if( crc & 0x8000 )
		{
			crc = (crc << 1) ^ 0x1021;
		}
		else
		{
			crc = crc << 1;
		}
	}
	
	return(crc);
}


//returns 0 if block is Bad
//returns 1 if block is OK
//returns 2 if transfer is finished
uint16_t xmodemDownloadBlock(TXmodemBlock *pBlock)
{
	uint16_t len;
	static uint8_t thisPacketWillBeAcked;
	
	do
	{
		
		
		if(xModemStarted==0)
		{
			debug_chr('C');
		}
		else
		{
			if(thisPacketWillBeAcked)
				debug_chr(ACK);
			else
				debug_chr(NAK);
		}
		
		uint8_t key;
		const uint32_t MAX_COUNT = 1000000;
		uint32_t counter=MAX_COUNT;
		len = 0;
		
		while(counter--)
		{
			key = debug_inkey();
			
			if(key) //received something
			{
				uint8_t *ptrToBlock;
				ptrToBlock = (void*) pBlock;
				
				ptrToBlock[0] = key; //copy it to packet as a start byte
				
				if(key == EOT)
				{
					debug_chr(NAK);
					key = debug_waitkey();
					if(key==EOT) debug_chr(ACK); else debug_chr(NAK);
					debug_msg("!!!");
					return(2);
				}
				
				//get next bytes of the packet
				uint16_t i;
				for(i=1;i<sizeof(TXmodemBlock);i++)
				{
					ptrToBlock[i] = debug_waitkey();
				}
				len = sizeof(TXmodemBlock);
				break;
			}
		}
		
	} while(len==0);
	
	xModemStarted = 1;
	
	if(pBlock->startByte == EOT)	//end of file
	{
		blockCounter = 1;
		xModemStarted = 0;
		thisPacketWillBeAcked = 0;
		debug_chr(NAK);
		uint8_t endByte = debug_waitkey();
		if(endByte == EOT)
		{
			debug_chr(ACK);
			return(OK);
		}
		else
		{
			return(FAIL);
		}
	}
	
	if( ( pBlock->startByte == CAN ) )	//file transfer interrupted by user
	{
		blockCounter = 1;
		xModemStarted = 0;
		debug_chr(CAN);
		debug_chr(CAN);
		debug_chr(CAN);
		return(FAIL);
	}
	
	//compute CRC
	uint8_t i;
	uint16_t crc=0;
	for(i=0;i<XMODEM_PAYLOAD_LENGTH;i++)
	{
		crc = xmodemNextByteCrc(crc, pBlock->payload[i]);
	}
	
	//swap bytes to big endian fromat
	crc = (crc >> 8) | (crc << 8);
	
	//compare received and computed CRCs
	if(crc != pBlock->checksum)
	{
		thisPacketWillBeAcked=0;
		return(FAIL);
	}
	else
	{
		thisPacketWillBeAcked=1;
	}
	
	//check packet number
	if( pBlock->packetNumber != (0xFF - pBlock->notPacketNumber) )
	{
		thisPacketWillBeAcked=0;
		return(FAIL);
	}
	
	//compare packet number with counter
	if(blockCounter != pBlock->packetNumber)
	{
		thisPacketWillBeAcked=0;
		return(FAIL);
	}
	else
	{
		thisPacketWillBeAcked=1;
		blockCounter++;
	}
	return(OK);
}



