#include "common.h"

//a 1MB buffer in SDRAM
volatile uint32_t aBuffer[0x100000] __attribute__((section(".buffers")));

//fast buffers in internal SRAM memory
uint32_t aFastBuffer0[0x100] __attribute__((section(".fast_data_0")));	//buffer is in SRAM0
uint32_t aFastBuffer1[0x100] __attribute__((section(".fast_data_1")));	//buffer is in SRAM1

//this buffer will be in .data section
uint32_t dataSectionBuffer[4] = {1,2,3,4};

//this buffer will be in .bss section
uint32_t bssSectionBuffer[4];


extern int _sfixed;
extern int _efixed;
extern int _srelocate;
extern int _erelocate;
extern int _szero;
extern int _ezero;


void init(void)
{
	// enable peripheral clock for PIO controller
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA) | (1<<AT91C_ID_PIOB) | (1<<AT91C_ID_PIOC);
	
	LEDS_INIT;
	
	debug_init_default();
}

int main( void )
{
	init();
	
	debug_msg("Buffer tester");
	
	debug_msg("aBuffer at address:           ");
		debug32_t((uint32_t)aBuffer);
	
	debug_msg("aFastBuffer0 at address:      ");
		debug32_t((uint32_t)aFastBuffer0);
	
	debug_msg("aFastBuffer1 at address:      ");
		debug32_t((uint32_t)aFastBuffer1);
	
	uint32_t *allocatedBuffer1 = malloc(0x1000*sizeof(uint32_t));
	uint8_t *allocatedBuffer2 = malloc(0x1000);
	
	debug_msg("allocatedBuffer1 at address:  ");
		debug32_t((uint32_t)allocatedBuffer1);
	debug_msg("allocatedBuffer2 at address:  ");
		debug32_t((uint32_t)allocatedBuffer2);
	
	debug_msg("bssSectionBuffer at address:  ");
		debug32_t((uint32_t)bssSectionBuffer);
	
	debug_msg("dataSectionBuffer at address: ");
		debug32_t((uint32_t)dataSectionBuffer);
	
	uint32_t anArrayOnStack[0x4000];
	debug_msg("anArrayOnStack at address:    ");
		debug32_t((uint32_t)anArrayOnStack);
	
	debug_msg("entering main loop");
	
	
	debug_msg("&_sfixed = "); debug32_t((uint32_t)&_sfixed);
	debug_msg("&_efixed = "); debug32_t((uint32_t)&_efixed);
	debug_msg("&_srelocate = "); debug32_t((uint32_t)&_srelocate);
	debug_msg("&_erelocate = "); debug32_t((uint32_t)&_erelocate);
	debug_msg("&_szero = "); debug32_t((uint32_t)&_szero);
	debug_msg("&_ezero = "); debug32_t((uint32_t)&_ezero);
	
	debug_value("fixed section length = ",(uint32_t)&_efixed - (uint32_t)&_sfixed);
	
	for(;;)
	{
		LED_G_ON;
		delay_ms(100);
		LED_G_OFF;
		LED_Y_ON;
		delay_ms(100);
		LED_Y_OFF;
		LED_R_ON;
		delay_ms(100);
		LED_R_OFF;
		LED_B_ON;
		delay_ms(100);
		LED_B_OFF;
	}
	return(0);
}

