#include "common.h"


void irq1Handler(void)
{
	debug_msg("IRQ1");
	LED_G_TOGGLE;
}


void fiqHandler(void)
{
	debug_msg("FIQ");
	LED_R_TOGGLE;
}


void pioIrqHandler(void)
{
	debug_msg("PIO IRQ");
	static uint32_t status;
	status = AT91C_BASE_PIOA->PIO_ISR;
	if(status & AT91C_PIO_PA7)
	{
		debug_txt(" PA7");
		LED_B_TOGGLE;
	}
}



void init(void)
{
	// enable peripheral clock for PIO controller
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA) | (1<<AT91C_ID_PIOB) | (1<<AT91C_ID_PIOC);
	LEDS_INIT;
	debug_init_default();
	
	debug_msg("Exceptions demo");
	
	enum {
		PRIORITY_DONT_CARE = 7,	//FIQ has the highest priority anyway
		PRIORITY_IRQ_1 = 0,
		PRIORITY_PIO = 1
	};
	
	//initialize FIQ
	AT91F_AIC_ConfigureIt(AT91C_BASE_AIC, AT91C_ID_FIQ, PRIORITY_DONT_CARE, AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE, fiqHandler);
	AT91F_AIC_EnableIt(AT91C_BASE_AIC,AT91C_ID_FIQ);
	
	//initialize IRQ1 (pushbutton)
	AT91F_AIC_ConfigureIt(AT91C_BASE_AIC, AT91C_ID_IRQ1, PRIORITY_IRQ_1, AT91C_AIC_SRCTYPE_EXT_NEGATIVE_EDGE, irq1Handler);
	AT91F_AIC_EnableIt(AT91C_BASE_AIC,AT91C_ID_IRQ1);
	AT91C_BASE_PIOC->PIO_PDR = AT91C_PC15_IRQ1;
	AT91C_BASE_PIOC->PIO_BSR = AT91C_PC15_IRQ1;
	
	//initialize IRQ from PIO Controller
	AT91F_AIC_ConfigureIt(AT91C_BASE_AIC, AT91C_ID_PIOA, PRIORITY_PIO, AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, pioIrqHandler);
	AT91F_AIC_EnableIt(AT91C_BASE_AIC,AT91C_ID_PIOA);
	AT91C_BASE_PIOA->PIO_PER = AT91C_PIO_PA7;
	AT91C_BASE_PIOA->PIO_ODR = AT91C_PIO_PA7;
	AT91C_BASE_PIOA->PIO_IER = AT91C_PIO_PA7;
	uint32_t dummy = AT91C_BASE_PIOA->PIO_ISR;
	dummy = dummy;
}



int main( void )
{
	init();
	
	debug_msg("entering main loop");
	
	for(;;)
	{
		LED_Y_TOGGLE;
		delay_ms(100);
	}
	return(0);
}

