//Coprocessor control stuff
//based on atmel example: AT91SAM9261-BasicMMU-ADS
//modified by RABW to be used with GCC compiler from WinARM

static inline unsigned int get_cp15(void)
{
	unsigned int value;
	asm volatile("mrc p15, 0, %0, c1, c0, 0" : "=r" (value));
	return value;
}

static inline void set_cp15(unsigned int value)
{
	asm volatile("mcr p15, 0, %0, c1, c0, 0" : : "r" (value));
}


// ****************************************************************************
// CP15 Register 0
// 	Read: ID code | cache type
//	Write: Unpredictable

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadIDCode
//* \brief Read ID code register
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadIDCode()
{
	register unsigned int id;
 	asm volatile("MRC p15, 0, %0, c0, c0, 0" : "=r" (id));
 	return id;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadCacheType
//* \brief Read cache type
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadCacheType()
{
	register unsigned int type;
 	asm volatile("MRC p15, 0, %0, c0, c0, 1" : "=r" (type));
 	return type;
}

// ****************************************************************************
// CP15 Register 1
// 	Read: Control
//	Write: Control

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadControl
//* \brief Read Control register
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadControl()
{
	register unsigned int ctl;
 	asm volatile("MRC p15, 0, %0, c1, c0, 0" : "=r" (ctl));
 	return ctl;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteControl
//* \brief Write Control register
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteControl(unsigned int ctl)
{
 	asm volatile("MCR p15, 0, %0, c1, c0, 0" : : "r" (ctl));
}

// ****************************************************************************
// CP15 Register 2
// 	Read: Translation table Base
//	Write: Translation table Base

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadTTB
//* \brief Read Translation table base register
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadTTB()
{
	register unsigned int ttb;
 	asm volatile("MRC p15, 0, %0, c2, c0, 0" : "=r" (ttb));
 	return ttb;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteTTB
//* \brief Write Translation table base  register
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteTTB(unsigned int ttb)
{
	ttb = ttb & 0xFFFFC000;	//align
 	asm volatile("MCR p15, 0, %0, c2, c0, 0" : : "r" (ttb));
}

// ****************************************************************************
// CP15 Register 3
// 	Read: Read domain access control
//	Write: Write domain access control

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadDomain
//* \brief Read domain access control
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadDomain()
{
	register unsigned int domain;
 	asm volatile("MRC p15, 0, %0, c3, c0, 0" : "=r" (domain));
 	return domain;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteDomain
//* \brief Write domain access control
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteDomain(unsigned int domain)
{
 	asm volatile("MCR p15, 0, %0, c3, c0, 0" : : "r" (domain));
}

// ****************************************************************************
// CP15 Register 5
// 	Read: Read Fault Status
//	Write: Write Fault Status

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadDataFSR
//* \brief Read data FSR value
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadDataFSR()
{
	register unsigned int dataFSR;
 	asm volatile("MRC p15, 0, %0, c5, c0, 0" : "=r" (dataFSR));
 	return dataFSR;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteDataFSR
//* \brief Write data FSR value
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteDataFSR(unsigned int dataFSR)
{
 	asm volatile("MCR p15, 0, %0, c5, c0, 0" : : "r" (dataFSR));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadPrefetchFSR
//* \brief Read prefetch FSR value
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadPrefetchFSR()
{
	register unsigned int prefetchFSR;
 	asm volatile("MRC p15, 0, %0, c5, c0, 1" : "=r" (prefetchFSR));
 	return prefetchFSR;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WritePrefetchFSR
//* \brief Write prefetch FSR value
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WritePrefetchFSR(unsigned int prefetchFSR)
{
 	asm volatile("MCR p15, 0, %0, c5, c0, 1" : : "r" (prefetchFSR));
}

// ****************************************************************************
// CP15 Register 6
// 	Read: Read Fault Address
//	Write: Write Fault Address

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadFAR
//* \brief Read FAR data
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadFAR()
{
	register unsigned int dataFAR;
 	asm volatile("MRC p15, 0, %0, c6, c0, 0" : "=r" (dataFAR));
 	return dataFAR;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteFAR
//* \brief Write FAR data
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteFAR(unsigned int dataFAR)
{
 	asm volatile("MCR p15, 0, %0, c6, c0, 0" : : "r" (dataFAR));
}

// ****************************************************************************
// CP15 Register 7
// 	Read: Unpredictable
//	Write: Cache operations

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateIDCache
//* \brief Invalidate ICache and DCache
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateIDCache()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c7, c7, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateICache
//* \brief Invalidate ICache
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateICache()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c7, c5, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateICacheMVA
//* \brief Invalidate ICache single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateICacheMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFFE0;
 	asm volatile("MCR p15, 0, %0, c7, c5, 1" : : "r" (mva));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_PrefetchICacheLine
//* \brief Prefetch ICache line (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_PrefetchICacheLine(unsigned int mva)
{
	mva = mva & 0xFFFFFFE0;
 	asm volatile("MCR p15, 0, %0, c7, c13, 1" : : "r" (mva));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateDCache
//* \brief Invalidate DCache
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateDCache()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c7, c6, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateDCacheMVA
//* \brief Invalidate DCache single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateDCacheMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFFE0;
 	asm volatile("MCR p15, 0, %0, c7, c6, 1" : : "r" (mva));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_CleanDCacheMVA
//* \brief Clean DCache single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_CleanDCacheMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFFE0;
 	asm volatile("MCR p15, 0, %0, c7, c10, 1" : : "r" (mva) );
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_CleanInvalidateDCacheMVA
//* \brief Clean and Invalidate DCache single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_CleanInvalidateDCacheMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFFE0;
 	asm volatile("MCR p15, 0, %0, c7, c14, 1" : : "r" (mva));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_CleanDCacheIDX
//* \brief Clean DCache single entry (using idx)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_CleanDCacheIDX(unsigned int idx)
{
	idx = idx & 0xFC0000E0;
 	asm volatile("MCR p15, 0, %0, c7, c10, 2" : : "r" (idx));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_CleanInvalidateDCacheIDX
//* \brief Clean and Invalidate DCache single entry (using idx)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_CleanInvalidateDCacheIDX(unsigned int idx)
{
	idx = idx & 0xFC0000E0;
 	asm volatile("MCR p15, 0, %0, c7, c14, 2" : : "r" (idx));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_TestCleanInvalidateDCache
//* \brief Test, Clean and Invalidate whole DCache
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_TestCleanInvalidateDCache()
{
	asm volatile
	(
		"tci_loop: MRC p15, 0, r15, c7, c10, 3 \n\t"
		"bne tci_loop \n\t"
	);
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_DrainWriteBuffer
//* \brief Drain Write Buffer
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_DrainWriteBuffer()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c7, c10, 4" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WaitForInterrupt
//* \brief Wait for interrupt
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WaitForInterrupt()
{
	register unsigned int sbz = 0;
	asm volatile("MCR p15, 0, %0, c7, c0, 4" : : "r" (sbz));
}

// ****************************************************************************
// CP15 Register 8
// 	Read: Unpredictable
//	Write: TLB operations

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateIDTLB
//* \brief Invalidate TLB(s)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateIDTLB()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c8, c7, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateITLB
//* \brief Invalidate I TLB
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateITLB()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c8, c5, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateITLBMVA
//* \brief Invalidate I TLB single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateITLBMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFE00;
 	asm volatile("MCR p15, 0, %0, c8, c5, 1" : : "r" (mva));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateDTLB
//* \brief Invalidate D TLB
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateDTLB()
{
	register unsigned int sbz = 0;
 	asm volatile("MCR p15, 0, %0, c8, c6, 0" : : "r" (sbz));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_InvalidateDTLBMVA
//* \brief Invalidate D TLB single entry (using MVA)
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_InvalidateDTLBMVA(unsigned int mva)
{
	mva = mva & 0xFFFFFE00;
 	asm volatile("MCR p15, 0, %0, c8, c6, 1" : : "r" (mva));
}


// ****************************************************************************
// CP15 Register 9
// 	Read: Cache lockdown
//	Write: Cache lockdown

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadDCacheLockdown
//* \brief Read D Cache lockdown
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadDCacheLockdown()
{
	register unsigned int idx;
 	asm volatile("MRC p15, 0, %0, c9, c0, 0" : "=r" (idx));
    return idx;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteDCacheLockdown
//* \brief Write D Cache lockdown
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteDCacheLockdown(
	unsigned int idx)
{
	asm volatile("MCR p15, 0, %0, c9, c0, 0" : : "r" (idx));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadICacheLockdown
//* \brief Read I Cache lockdown
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadICacheLockdown()
{
	register unsigned int idx;
	asm volatile("MRC p15, 0, %0, c9, c0, 1" : "=r" (idx));
	return idx;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteICacheLockdown
//* \brief Write I Cache lockdown
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteICacheLockdown(
	unsigned int idx)
{
	asm volatile("MCR p15, 0, %0, c9, c0, 1" : : "r" (idx));
}

// ****************************************************************************
// CP15 Register 10
// 	Read: TLB lockdown
//	Write: TLB lockdown

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadDTLBLockdown
//* \brief Read D TLB lockdown
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadDTLBLockdown()
{
	register unsigned int lockdown;
 	asm volatile("MRC p15, 0, %0, c10, c0, 0" : "=r" (lockdown));
    return lockdown;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteDTLBLockdown
//* \brief Write D TLB lockdown
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteDTLBLockdown(
	unsigned int lockdown)
{
	asm volatile("MCR p15, 0, %0, c10, c0, 0" : : "r" (lockdown));
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadITLBLockdown
//* \brief Read I TLB lockdown
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadTLBLockdown()
{
	register unsigned int lockdown;
	asm volatile("MRC p15, 0, %0, c10, c0, 0" : "=r" (lockdown));
	return lockdown;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteITLBLockdown
//* \brief Write I TLB lockdown
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteTLBLockdown(
	unsigned int lockdown)
{
	asm volatile("MCR p15, 0, %0, c10, c0, 0" : : "r" (lockdown));
}

// ****************************************************************************
// CP15 Register 13
// 	Read: Read FCSE PID
//	Write: Write FCSE PID

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_ReadFCSEPID
//* \brief Read FCSE PID
//*----------------------------------------------------------------------------
static inline unsigned int AT91F_ARM_ReadFCSEPID()
{
	register unsigned int pid;
 	asm volatile("MRC p15, 0, %0, c13, c0, 0" : "=r" (pid));
 	return pid;
}

//*----------------------------------------------------------------------------
//* \fn    AT91F_ARM_WriteFCSEPID
//* \brief Write FCSE PID
//*----------------------------------------------------------------------------
static inline void AT91F_ARM_WriteFCSEPID(unsigned int pid)
{
	pid = pid & 0xFF000000;
 	asm volatile("MCR p15, 0, %0, c13, c0, 0" : : "r" (pid));
}

