#include "common.h"
#include "sd.h"

#define FILE_NAME_LEN		(_MAX_LFN+1)
#define FILE_BUFFER_SIZE	200

static char fileName[FILE_NAME_LEN] __attribute__((section(".buffers")));
static char textBuffer[FILE_BUFFER_SIZE] __attribute__((section(".buffers")));;
#define BUF_SIZE		(10*1024*1024)
#define BUF_SIZE_WORDS	(BUF_SIZE / 4)
static int32_t buffer[BUF_SIZE_WORDS] __attribute__((section(".buffers")));
static FATFS fs __attribute__((section(".buffers")));
static FIL file __attribute__((section(".buffers")));

static uint32_t rttcRtmrContents;




void init(void)
{
	// enable peripheral clock for PIO controller
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA) | (1<<AT91C_ID_PIOB) | (1<<AT91C_ID_PIOC);
	LEDS_INIT;
	debug_init_default();
	
	debug_msg("AT91SAM9260 Empty Project");
	
	debug_msg("initializing filesystem");
	//if(sdInit()==OK) debug_msg("sdInit OK!"); else debug_msg("sdInit FAILED!");
	FRESULT res = f_mount(0,&fs);
	debug_msg("result: "); debug8_t(res);
}


void timerStart(void)
{
	//save RTMR contents to a variable
	rttcRtmrContents = AT91C_BASE_RTTC->RTTC_RTMR;
	
	//reset timer, set prescaler
	AT91C_BASE_RTTC->RTTC_RTMR = AT91C_RTTC_RTTRST | 3277;	//about 10 ticks per 1 sec
}


uint32_t timerStop(void)
{
	//restore RTMR contents from variable
	uint32_t time = AT91C_BASE_RTTC->RTTC_RTVR;
	AT91C_BASE_RTTC->RTTC_RTMR = rttcRtmrContents;
	return(time);
}





int main( void )
{
	init();
	
	debug_msg("entering main loop");
	
	//debug_msg("buffer start at "); debug32_t((uint32_t)buffer);
	debug_msg("RTT_MR = "); debug32_t(AT91C_BASE_RTTC->RTTC_RTMR);
	
	for(;;)
	{
		uint8_t key = debug_inkey();
		switch(key)
		{
			case 'i':
			{
				debug_msg("reinitializing filesystem");
				//if(sdInit()==OK) debug_msg("sdInit OK!"); else debug_msg("sdInit FAILED!");
				FRESULT res = f_mount(0,&fs);
				debug_msg("result: "); debug8_t(res);
				break;
			}
			case 's':
			{
				debug_msg("reading card size:");
				uint32_t numberOfBlocks;
				uint16_t blockSize;
				sdGetSizeInfo(&numberOfBlocks,&blockSize);
				debug_msg(" * number of blocks: 0x"); debug32_t(numberOfBlocks);
				debug_msg(" * blocks size:      "); debug_word(blockSize); debug_txt(" B");
				debug_msg(" * card capacity:    "); debug_word(numberOfBlocks*blockSize/1024/1024); debug_txt(" MB");
				break;
			}
			case 'l':
			{
				DIR dir;
				FILINFO fileInfo;
				char longFileName[FILE_NAME_LEN];
				debug_msg("listing files in a directory, enter dir name: ");
				debug_input_string(fileName,FILE_NAME_LEN);
				FRESULT res = f_opendir(&dir, fileName);
				
				if(res != FR_OK)
				{
					debug_msg("f_opendir returned bad status: ");
					debug8_t(res);
					break;
				}
				
				//set LFN parameters
				fileInfo.lfname = longFileName;		//provide buffer
				fileInfo.lfsize = _MAX_LFN;	//set buffer length
				
				uint32_t counter = 0;
				
				while(1)
				{
					res = f_readdir(&dir, &fileInfo);
					if(res != FR_OK) break;
					if(fileInfo.fname[0]==0) break;
					
					debug_value("",++counter);
						debug_txt(": ");
						if(fileInfo.fattrib & AM_DIR)
						{
							debug_txt("[Dir ] ");
						}
						else
						{
							debug_txt("[File] ");
						}
						debug_txt(fileInfo.fname);
						debug_txt(" LFN: ");
					
					//display file/dir name
					if(fileInfo.lfname[0]==0)
					{
						debug_txt("[no LFN]");
					}
					else
					{
						debug_txt(fileInfo.lfname);
					}
				}
				debug_msg("done");
				break;
			}
			case 'w':
			{
				FRESULT res;
				debug_msg("write a file, enter filename: ");
				debug_input_string(fileName,FILE_NAME_LEN);
				res = f_open(&file,fileName,FA_WRITE | FA_CREATE_ALWAYS);
				debug_msg("f_open returned "); debug8_t(res);
				debug_msg("enter a text: ");
				uint16_t len = debug_input_string(textBuffer,FILE_BUFFER_SIZE);
				debug_msg("writing to file...");
				uint32_t written;
				res = f_write(&file,textBuffer,len,&written);
				debug_msg("f_write returned "); debug8_t(res);
				debug_txt(", bytes written: "); debug16_t(written);
				res = f_close(&file);
				debug_msg("f_close returned "); debug8_t(res);
				break;
			}//case 'w':
			case 'r':
			{
				FRESULT res;
				debug_msg("read a file, filename: ");
				debug_input_string(fileName,FILE_NAME_LEN);
				res = f_open(&file,fileName,FA_READ);
				debug_msg("f_open returned "); debug8_t(res);
				uint32_t read;
				res = f_read(&file,textBuffer,FILE_BUFFER_SIZE,&read);
				debug_msg("read returned "); debug8_t(res);
				debug_txt(", bytes read: "); debug16_t(read);
				debug_msg("file contents:");
				debug_txt_limit((char*)textBuffer,read);
				res = f_close(&file);
				debug_msg("f_close returned "); debug8_t(res);
				break;
			}//case 'r':
			case 't':
			{
				debug_msg("testing write/read speed. Enter file name to create: ");
				uint8_t len = debug_input_string(fileName,FILE_NAME_LEN);
				if(len==0) {debug_msg("aborted"); break;}
				
				//create some contents in buffer
				
				debug_msg("preparing buffer contents...");
				for(uint32_t i=0;i<BUF_SIZE_WORDS;i++)
				{
					buffer[i] = -i;
				}
				debug_txt("OK");
				
				//write buffer to a file
				
				debug_msg("opening a file to write...");
				FRESULT res = f_open(&file,fileName,FA_WRITE | FA_CREATE_ALWAYS);
				if(res == FR_OK)
				{
					debug_txt("OK");
				}
				else
				{
					debug_txt("write error, res: "); debug8_t(res);
					break;
				}
				debug_msg("writing 0x"); debug8_t(BUF_SIZE / 1024 / 1024); debug_txt(" MB of data...");
				UINT counter;	//stores number of bytes written/read
				timerStart();
				res = f_write(&file,buffer,BUF_SIZE,&counter);
				uint32_t time = timerStop();
				if(res == FR_OK)
				{
					debug_txt("OK");
				}
				else
				{
					debug_txt("write error, res: "); debug8_t(res);
					break;
				}
				debug_txt(", bytes written: "); debug32_t(counter);
				debug_msg("time: "); debug_word(time); debug_txt(" * 0,1s");
				debug_msg("write speed is: "); debug_word( 10*counter/time/1024 ); debug_txt(" KB/s");
					debug_txt(" = "); debug_word(10*counter/time/1024/1024); debug_txt(" MB/s");
				debug_msg("closing file...");
				f_close(&file);
				debug_txt("OK");
				
				//read data from file
				
				debug_msg("ropening file...");
				res = f_open(&file,fileName,FA_READ);
				if(res == FR_OK)
				{
					debug_txt("OK");
				}
				else
				{
					debug_txt("error, res: "); debug8_t(res);
				}
				debug_msg("Reading data...");
				timerStart();
				res = f_read(&file,buffer,BUF_SIZE,&counter);
				time = timerStop();
				if(res == FR_OK)
				{
					debug_txt("OK");
				}
				else
				{
					debug_txt("error, res: "); debug8_t(res);
				}
				debug_txt(", bytes read: "); debug32_t(counter);
				debug_msg("time: "); debug_word(time); debug_txt(" * 0,1s");
				debug_msg("read speed is: "); debug_word( 10*counter/time/1024 ); debug_txt(" KB/s");
					debug_txt(" = "); debug_word(10*counter/time/1024/1024); debug_txt(" MB/s");
				debug_msg("closing file...");
				f_close(&file);
				debug_txt("OK");
				
				//test data
				debug_msg("verifying data...");
				uint32_t errors = 0;
				for(uint32_t i=0;i<BUF_SIZE_WORDS;i++)
				{
					if(buffer[i]!=(-i))
					{
						debug_msg(" error at i="); debug32_t(i); debug_txt(", value expected:");
							debug32_t(-i); debug_txt(", value read: "); debug32_t(buffer[i]);
						errors++;
						break;
					}
				}
				debug_msg("verify done, number of errors: "); debug32_t(errors);
				
				break;
			}//case 't':
		}//switch(key)
		LED_B_TOGGLE;
		delay_ms(100);
	}
	return(0);
}

