#include "common.h"
#include "i2c.h"

//mcu dependent defs:
#define I2C_PIO					AT91C_BASE_PIOB
#define I2C_SDA_MASK			(1<<11)
#define I2C_SCL_MASK			(1<<10)

#define I2C_MASK				(I2C_SDA_MASK|I2C_SCL_MASK)

//pin defs
#define I2C_PIN_INIT			\
	{\
	I2C_PIO->PIO_PER = I2C_MASK;\
	I2C_PIO->PIO_ODR = I2C_SDA_MASK;\
	I2C_PIO->PIO_OER = I2C_SCL_MASK;\
	I2C_PIO->PIO_SODR = I2C_SCL_MASK;\
	I2C_PIO->PIO_CODR = I2C_SDA_MASK;\
	}
#define I2C_SDA_IS_HIGH			(I2C_PIO->PIO_PDSR & I2C_SDA_MASK)
#define I2C_SDA_1				I2C_PIO->PIO_ODR = I2C_SDA_MASK
#define I2C_SDA_0				I2C_PIO->PIO_OER = I2C_SDA_MASK
#define I2C_SCL_1				I2C_PIO->PIO_SODR = I2C_SCL_MASK
#define I2C_SCL_0				I2C_PIO->PIO_CODR = I2C_SCL_MASK



void i2cDelay(void);


void i2cStart(void)
{
	I2C_PIN_INIT;
	I2C_SDA_1;
	i2cDelay();
	I2C_SCL_1;
	i2cDelay();
	I2C_SDA_0;
	i2cDelay();
	I2C_SCL_0;
	i2cDelay();
}


void i2cStop(void)
{
	I2C_SDA_0;
	i2cDelay();
	I2C_SCL_1;
	i2cDelay();
	I2C_SDA_1;
	i2cDelay();
}


uint8_t i2cRead(uint8_t ack)
{
	uint8_t i;
	uint8_t data=0;
	I2C_SDA_1;
	for(i=0;i<8;i++)
	{
		I2C_SCL_0;
		i2cDelay();
		I2C_SCL_1;
		i2cDelay();
		data = data << 1;
		if(I2C_SDA_IS_HIGH) data |= 1;
	}
	I2C_SCL_0;
	if(ack==I2C_ACK) {I2C_SDA_0;} else {I2C_SDA_1;}
	i2cDelay();
	I2C_SCL_1;
	i2cDelay();
	I2C_SCL_0;
	return(data);
}


uint8_t i2cWrite(uint8_t data)
{
	uint8_t i, ack;
	for(i=0;i<8;i++)
	{
		if(data&0x80) {I2C_SDA_1;} else {I2C_SDA_0;}
		data = data << 1;
		i2cDelay();
		I2C_SCL_1;
		i2cDelay();
		I2C_SCL_0;
		i2cDelay();
	}
	//ack
	I2C_SDA_1;
	i2cDelay();
	I2C_SCL_1;
	i2cDelay();
	if(I2C_SDA_IS_HIGH) {ack=I2C_NACK;} else {ack=I2C_ACK;}
	I2C_SCL_0;
	i2cDelay();
	return(ack);
}


void i2cDelay(void)
{
	delay_ms(1);
}


void i2cInit(void)
{
	I2C_PIN_INIT;
	i2cStop();
}




