#include "common.h"
#include "lcd.h"
#include "camera.h"
#include "bitmap.h"


#define FILE_NAME_LEN		(_MAX_LFN+1)

static volatile uint32_t image[CAM_IMAGE_Y_SIZE*CAM_IMAGE_X_SIZE/2]__attribute__((section(".buffers")));
static volatile uint16_t preview[CAM_PREVIEW_Y_SIZE*CAM_PREVIEW_X_SIZE]__attribute__((section(".buffers")));
static volatile uint16_t lcdPreview[LCD_Y_SIZE][LCD_X_SIZE];

static char fileName[FILE_NAME_LEN] __attribute__((section(".buffers")));
static FATFS fs __attribute__((section(".buffers")));


void init(void)
{
	// enable peripheral clock for PIO controller
	AT91C_BASE_PMC->PMC_PCER = (1<<AT91C_ID_PIOA) | (1<<AT91C_ID_PIOB) | (1<<AT91C_ID_PIOC);
	LEDS_INIT;
	debug_init_default();
	
	debug_msg("AT91SAM9260 camera demo");
	
	debug_msg("initializing filesystem...");
	//if(sdInit()==OK) debug_msg("sdInit OK!"); else debug_msg("sdInit FAILED!");
	FRESULT res = f_mount(0,&fs);
	debug_txt("result: "); debug8_t(res);
	
	lcdInit();
	
	camInit((void*)preview,(void*)image);
}


//simple util to convert integer (param i) into ASCII string stored in buf
//converts values from 0 to 999 into "000" to "999" strings
//returns nonzero if conversion was OK, else returns 0
static uint32_t itoa(char *buf, uint16_t i)
{
	if(i > 999) return FAIL;
	buf[0] = i/100 + '0';
	buf[1] = (i%100)/10 + '0';
	buf[2] = i%10 + '0';
	buf[3] = 0;
	return(OK);
}


//base: pointer to base of file name. Must be max 5 chars long
//ext: pointer to extension of file name. Must be max 3 chars long
//outputName: fileName of non-existing file
//returns OK if free file name was found or FAIL if not
uint32_t fsFindFreeFileName(const char *base, const char *ext, char *outputName)
{
	static FILINFO info;
	uint16_t nameCounter = 0;
	FRESULT res;
	
	do
	{
		//debug_msg("new filename test loop");
		//prepare file name to test
		
		//get length of base
		uint32_t len = strlen(base);
		if(len > 5) return(FAIL);
		
		//copy base to file name to be tested
		strcpy(outputName,base);
		
		//convert file number into string
		itoa(&outputName[len],nameCounter);
		
		//dot
		strcat(outputName,".");
		
		//extension
		strcat(outputName,ext);
		
		//display name to be tested
		//debug_msg("trying name: "); debug_txt(outputName);
		
		//test the new name
		res = f_stat((const char*)outputName,&info);
		
		//increment file number to be tested
		nameCounter++;
		if(nameCounter > 999) {debug_msg("fsFindFreeFileName: FAIL, couldn't find any free file name"); return(FAIL);}
		
	}while(res == FR_OK);	//exit if file was not found (doesn't exist)
	
	//display file name to be used (option)
	//debug_msg("found free name: "); debug_txt(outputName);
	
	return(OK);
}


void previewToLcd(void)
{
	uint16_t *pImage;
	pImage = (void*)preview;
	uint32_t line = CAM_PREVIEW_Y_SIZE,column;
	while(line--)
	{
		for(column = CAM_PREVIEW_X_SIZE; column > 0; --column)	//yes, a little bugfix is here
		{
			lcdPreview[line][column] = *pImage++;
		}
	}
}


void camTestProc(void)
{
	uint8_t key = debug_inkey();
	switch(key)
	{
		case 't':
		{
			debug_msg("test pattern, enter number: ");
			uint8_t pattern = debug_input_hex(0);
			camShowTestScreen(pattern);
			break;
		}
		case 'i':
		{
			debug_msg("reinitializing filesystem");
			FRESULT res = f_mount(0,&fs);
			debug_txt("result: "); debug8_t(res);
			break;
		}
	}
}



int main( void )
{
	init();
	
	debug_msg("entering main loop");
	
	for(;;)
	{
		previewToLcd();
		lcdWriteRgb555((void*)lcdPreview);
		lcdUpdate();
		camTestProc();
		if(SW1_PRESSED)
		{
			debug_msg("capture still image");
			camCaptureStillImage();
			if(fsFindFreeFileName("IMG","BMP",fileName)!=OK) break;
			if(bmpSaveYCbCr((void*)&image[0],CAM_IMAGE_X_SIZE,CAM_IMAGE_Y_SIZE,fileName)!=OK)
			{
				debug_msg("bmpSave error");
			}
			else
			{
				debug_msg("bmpSave OK");
			}
		}
	}
	return(0);
}

